@@ -1,9 +1,6 @@
-use funspace::{
-    cheb_dirichlet, fourier_r2c, BaseAll, BaseC2c, BaseR2c, BaseR2r, BaseSpace, Basics, FloatNum,
-    LaplacianInverse, Space2,
-};
+use funspace::{cheb_dirichlet, fourier_r2c, BaseKind, BaseSpace, FloatNum, ScalarNum, Space2};
+use ndarray::Ix;
 use ndarray::{prelude::*, Data};
-use ndarray::{Ix, ScalarOperand};
 use num_complex::Complex;
 use std::convert::TryInto;
 
@@ -31,8 +28,8 @@ pub type Field2<T2, S> = FieldBase<f64, f64, T2, S, 2>;
 
 impl<A, T1, T2, S, const N: usize> FieldBase<A, T1, T2, S, N>
 where
-    A: FloatNum,
-    Complex<A>: ScalarOperand,
+    A: FloatNum + ScalarNum,
+    Complex<A>: ScalarNum,
     S: BaseSpace<A, N, Physical = T1, Spectral = T2>,
 {
     pub fn new(space: &S) -> Self {
@@ -104,54 +101,49 @@ where
         })
     }
 
+    #[allow(unreachable_patterns)]
     pub fn ingredients_for_hholtz(&self, axis: usize) -> (Array2<A>, Array2<A>, Option<Array2<A>>) {
-        let x = &self.space.base_all()[axis];
-        let mass = x.mass();
-        let lap = x.laplace();
-        let peye = x.laplace_inv_eye();
-        let pinv = peye.dot(&x.laplace_inv());
+        let kind = self.space.base_kind(axis);
+        let mass = self.space.mass(axis);
+        let lap = self.space.laplace(axis);
 
-        // Matrices
-        let (mat_a, mat_b) = match x {
-            BaseAll::BaseR2r(ref b) => match b {
-                BaseR2r::Chebyshev(_) => {
-                    let mass_sliced = mass.slice(s![.., 2..]);
-                    (pinv.dot(&mass_sliced), peye.dot(&mass_sliced))
-                }
-                BaseR2r::CompositeChebyshev(_) => (pinv.dot(&mass), peye.dot(&mass)),
-            },
-            BaseAll::BaseR2c(ref b) => match b {
-                BaseR2c::FourierR2c(_) => (mass, lap),
-            },
-            BaseAll::BaseC2c(ref b) => match b {
-                BaseC2c::FourierC2c(_) => (mass, lap),
-            },
-        };
-        // Preconditioner (optional)
-        let precond = match x {
-            BaseAll::BaseR2r(ref b) => match b {
-                BaseR2r::Chebyshev(_) | BaseR2r::CompositeChebyshev(_) => Some(pinv),
-            },
-            BaseAll::BaseR2c(_) | BaseAll::BaseC2c(_) => None,
-        };
-        (mat_a, mat_b, precond)
+        // Matrices and optional preconditioner
+        match kind {
+            BaseKind::Chebyshev => {
+                let peye = self.space.laplace_inv_eye(axis);
+                let pinv = peye.dot(&self.space.laplace_inv(axis));
+                let mass_sliced = mass.slice(s![.., 2..]);
+                (pinv.dot(&mass_sliced), peye.dot(&mass_sliced), Some(pinv))
+            }
+            BaseKind::ChebDirichlet | BaseKind::ChebNeumann | BaseKind::ChebDirichletNeumann => {
+                let peye = self.space.laplace_inv_eye(axis);
+                let pinv = peye.dot(&self.space.laplace_inv(axis));
+                (pinv.dot(&mass), peye.dot(&mass), Some(pinv))
+            }
+            BaseKind::FourierR2c | BaseKind::FourierC2c => (mass, lap, None),
+            _ => panic!("No ingredients found for Base kind: {}!", kind),
+        }
     }
 
+    #[allow(unreachable_patterns)]
     pub fn ingredients_for_poisson(
         &self,
         axis: usize,
     ) -> (Array2<A>, Array2<A>, Option<Array2<A>>, bool) {
-        let x = &self.space.base_all()[axis];
-
         // Matrices and preconditioner
         let (mat_a, mat_b, precond) = self.ingredients_for_hholtz(axis);
 
         // Boolean, if laplacian is already diagonal
         // if not, a eigendecomposition will diagonalize mat a,
         // however, this is more expense.
-        let is_diag = match x {
-            BaseAll::BaseR2r(_) => false,
-            BaseAll::BaseR2c(_) | BaseAll::BaseC2c(_) => true,
+        let kind = self.space.base_kind(axis);
+        let is_diag = match kind {
+            BaseKind::Chebyshev
+            | BaseKind::ChebDirichlet
+            | BaseKind::ChebNeumann
+            | BaseKind::ChebDirichletNeumann => false,
+            BaseKind::FourierR2c | BaseKind::FourierC2c => true,
+            _ => panic!("No ingredients found for Base kind: {}!", kind),
         };
 
         (mat_a, mat_b, precond, is_diag)
